//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : compensation_Mayn.c
//* Object              : Mayn application written in C
//* Creation            : S.CADENE   2nd November 2004
//* Vers                : V1.0
//* Aim                 : This program is using RTT periperal and compensation process
//*						: to obtain software RTC. The result is sent to DBGU port COM
//*						: in following format [MM DD YY HH:MM:SS].
//*                     : Note: Refer to Atmel Application Note in term
//*						: of documentation around the compensation process.
//*----------------------------------------------------------------------------


#include "Board.h"
#include "compensation_main.h"
#include <stdio.h>


// Preset a valid date
char refdate[7]={'8','0','0','1','0','1','0'};

// Used to display message through the Port COM
char message[80];

//*--------------------------------------------------------------------------------------
//* Function Name       : RTPPRES_Correction
//* Object              : Compensation process
//* Input Parameters    : Delay for the next compensation in second.
//* Output Parameters   : none.
//*--------------------------------------------------------------------------------------
void RTPPRES_Correction (int next_compensation_delay)
{

  AT91PS_PMC     pPMC = AT91C_BASE_PMC;

  unsigned int freq_rc, n;

/* FIRST STEP: We are going to start the embedded PMC process called Timing Interval Acquisition
in application note. For that, we have to stop and start the Mayn Oscillator */
  	// Switch to Slow
        pPMC->PMC_MCKR = (0x4 | AT91C_PMC_CSS_SLOW_CLK);
        while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

        // Modify PRES value down to 0
        pPMC->PMC_MCKR = AT91C_PMC_CSS_SLOW_CLK;
        while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

        // Stop the PLL
        pPMC->PMC_PLLR = 0;

        // Stop the Main Oscillator
        pPMC->PMC_MOR = 0;

        // Start the Main Oscillator
        pPMC->PMC_MOR = (( AT91C_CKGR_OSCOUNT & (0x06 <<8) | AT91C_CKGR_MOSCEN ));

        // Waiting necessary Main Oscillator startup time
        while(!(pPMC->PMC_SR & AT91C_PMC_MOSCS));

        // Switch to the Mayn Oscillator clock
        pPMC->PMC_MCKR = AT91C_PMC_CSS_MAIN_CLK;
        while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

        // Start the PLL and waiting necessary settling time
        // The PLLCOUNT value has been calculated according to the following parameters:
        /* PLL Filter components R=1.5k, C1=10 nF, C2=1nF, Fn=18kHz,
        Settling Time (max)= 855 s, RC Oscillator (max)=42 kHz      */
        pPMC->PMC_PLLR = ((AT91C_CKGR_DIV & 0x0E) |
                         (AT91C_CKGR_PLLCOUNT & (0x28 << 8)) |
                         (AT91C_CKGR_MUL & (0x48 << 16)));
        while(!(pPMC->PMC_SR & AT91C_PMC_LOCK));

        // Switch to the PLL Clock
        // Set the PRES field to divide the PLL output by  2
		pPMC->PMC_MCKR = AT91C_PMC_PRES_CLK_2 ;
        while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

        // Switch to PLL output. After this step we're running from the PLL
        pPMC->PMC_MCKR = ((pPMC->PMC_MCKR)|AT91C_PMC_CSS_PLL_CLK);
         while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

/* SECOND STEP: Evaluating real RC-Oscillator working frequency according to the Main Oscillator */

         freq_rc =  (count_period/((*AT91C_CKGR_MCFR & AT91C_CKGR_MAINF)));

        /* Waiting new end of cycle before to modify the RTPPRES field vs. MAINF value
         (take a look at dedicated ATMEL' s Application Note)
        !! IMPORTANT !!: Because there is this constraint, this task should be used in interrupt mode
        In this example, we're using pulling method (It is not the best choice, of course)
        to wait next event just before upgrade RTT prescaler value */
        n = *AT91C_RTTC_RTVR;

        while (n == *AT91C_RTTC_RTVR);
//			    	/ \
//			    	 |
//			The user has to take care in having the smallest delay between the end of the while
//			and the *AT91C_RTTC_RTMR modification. In best case, this part must work from the internal
//			RAM in order to fetch without wait state @ full speed.
//			    	 |
//			    	\ /
        *AT91C_RTTC_RTMR  =freq_rc;

        // Preset the next compensation event
        *AT91C_RTTC_RTAR = (n) + next_compensation_delay;

}
//*--------------------------------------------------------------------------------------
//* Function Name       : Date_preset
//* Object              : Capture user data through the Hyperterminal
//*						: and set the references in term of Date and Time
//* Input Parameters    : None
//* Output Parameters   : None
//*--------------------------------------------------------------------------------------
unsigned int  Date_preset (void)
{

unsigned int Data;
unsigned int Second;

	// Date Capture
   AT91F_DBGU_Printk("\n\rEnter Year (AA):");
   AT91F_DBGU_scanf("%d",&Data);
   refdate[0]=Data/10 +'0';
   refdate[1]= Data - ((refdate[0]-'0')*10) +'0';
   AT91F_DBGU_Printk("\n\rEnter Month (MM):");
   AT91F_DBGU_scanf("%d",&Data);
   refdate[2]=Data/10+'0';
   refdate[3]=Data - ((refdate[2]-'0')*10) +'0';
   AT91F_DBGU_Printk("\n\rEnter Day   (DD):");
   AT91F_DBGU_scanf("%d",&Data);
   refdate[4]=Data/10+'0';
   refdate[5]=Data - ((refdate[4]-'0')*10) +'0';

	// Time Capture and conversion in second
   AT91F_DBGU_Printk("\n\rEnter  (HH):");
   AT91F_DBGU_scanf("%d",&Data);

   Second=Data*3600;

   AT91F_DBGU_Printk("\n\rEnter  (MM):");
   AT91F_DBGU_scanf("%d",&Data);

   Second+=Data*60;

   AT91F_DBGU_Printk("\n\rEnter  (SS):");
   AT91F_DBGU_scanf("%d",&Data);
   AT91F_DBGU_Printk("\n\r");
   Second+=Data;

   //Convert date to second with starting point 01/01/1980
   Second += date_to_days(refdate) * 86400;

   return (Second);
}

//*--------------------------------------------------------------------------------------
//* Function Name       : DateTime_Request
//* Object              : Return time and date
//* Input Parameters    : None
//* Output Parameters   : Update MM/DD/YY and HH/MM/SS
//*--------------------------------------------------------------------------------------
void DateTime_request (unsigned int second)
{
  unsigned int number_day;
  char displaydate[7];
  char displaytime[5];

    // Number of second + elapsed second (from RTT information)
    second += *AT91C_RTTC_RTVR;

    //
	number_day = second / 86400;
	days_to_date(number_day, displaydate);

	second -= (number_day * 86400);

	minute_to_hour((second/60), displaytime);

	//Date Displaying
	sprintf(message,"\n\rDate MM DD YY HH:MM:SS");
    AT91F_DBGU_Printk(message);
    sprintf(message," %d" ,displaydate[2] -'0');
    AT91F_DBGU_Printk(message);
    sprintf(message,"%d-",displaydate[3] -'0');
    AT91F_DBGU_Printk(message);
    sprintf(message,"%d" ,displaydate[4] -'0');
    AT91F_DBGU_Printk(message);
    sprintf(message,"%d-",displaydate[5] -'0');
    AT91F_DBGU_Printk(message);
    sprintf(message,"%d" ,displaydate[0] -'0');
    AT91F_DBGU_Printk(message);
    sprintf(message,"%d" ,displaydate[1] -'0');
    AT91F_DBGU_Printk(message);

//Time Displaying
    sprintf(message," %d" ,displaytime[0] -'0');
    AT91F_DBGU_Printk(message);
    sprintf(message,"%d:",displaytime[1] -'0');
    AT91F_DBGU_Printk(message);
    sprintf(message,"%d" ,displaytime[2] -'0');
    AT91F_DBGU_Printk(message);
    sprintf(message,"%d:",displaytime[3] -'0');
    AT91F_DBGU_Printk(message);
    second %= 60;
    sprintf(message,"%d\n\r ",second);
    AT91F_DBGU_Printk(message);
}

//*--------------------------------------------------------------------------------------
//* Function Name       : Mayn
//* Object              : Software entry point
//* Input Parameters    : none.
//* Output Parameters   : none.
//*--------------------------------------------------------------------------------------
int main()
{
unsigned int ref_time_InSecond;

   AT91F_DBGU_Init();
   AT91F_DBGU_Printk("\n\rATMEL TEST RC 2005\n\r");

   // The ref_time_InSecond must be hold all the time in application
   ref_time_InSecond = Date_preset();

        // First Step: Reset RTT and Start it
		*AT91C_RTTC_RTMR = 0x40000;
        RTPPRES_Correction (compens_delay);

        while(1)
        {
            // Second Step:
            if (((*AT91C_RTTC_RTSR) & AT91C_RTTC_ALMS) == AT91C_RTTC_ALMS)
            {
              RTPPRES_Correction (compens_delay);
              DateTime_request (ref_time_InSecond);
            }

       }
}
//* End
