//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : utils.c
//* Object              : Time and date utility functions
//* Creation            : S.CADENE   November 2005
//*----------------------------------------------------------------------------

#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include "Board.h"


static unsigned int day_start_year[12] =
    {
        0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
    };


int is_date_valid(const char *data_string);


/************************************************************************************/
/* String Management Function *******************************************************/
/************************************************************************************/

char *strncpy0(char *destination, const char *source, int n)
/* This function is working like strncpy in LLIBCA.LIB but only with n character */
{
    strncpy(destination, source, n);
    destination[n] = '\0';
    return destination;
}

/************************************************************************************/
/* Time and Date  Management Functions **********************************************/
/************************************************************************************/


void minute_to_hour(unsigned short minutes, char *hour)
/* Convert minutes to hour. The resulting format will be
   HHMM. */
{
    if(minutes > 1440)
        minutes = 0;
    hour[0] = minutes / 600 + '0';
    minutes %= 600;
    hour[1] = minutes / 60 + '0';
    minutes %= 60;
    hour[2] = minutes / 10 + '0';
    minutes %= 10;
    hour[3] = minutes + '0';
    hour[4] = 0;
}


unsigned int date_to_days(const char *date)
/* Return the elapsed days since 01/01/1980.
The expected format is YYMMDD */
{
unsigned int result;
unsigned int year;
unsigned char tmp;
	if(!is_date_valid(date) )
		return 0;
    if(date[0] > '7')
    {
        year = 1900;
    }
    else
    {
        year = 2000;
    }
    year += (date[0] - '0') * 10 + date[1] - '0';
    result = 1461 * ( (year - 1980) / 4);         /* 1461 = 4 * 365 + 1 */
    tmp = (year - 1980) % 4;
    switch(tmp)
    {
        case 0 :
            break;
        case 1 :
            result += 366;
            break;
        case 2 :
            result += 366 + 365;
            break;
        case 3 :
            result += 366 + 365 + 365;
            break;
    }
    tmp = (date[2] - '0') * 10 + date[3] - '0';  /* No de month */
    result += day_start_year[tmp - 1];
    if( ( (year % 4) == 0) && (tmp > 2) )
        result++;                      /* si year bissextile */
    result += (date[4] - '0') * 10 + date[5] - '1';  /* No de day */
    return result;
}


void days_to_date(unsigned short days, char *date)
/* Inverse function of previous. The year will be from 1980 up to 2079.
Return the date in YYMMDD format */

{
unsigned int year;
unsigned char month;
    if(days > 36524)
        days = 0;
    year = 1980 + (4 * (days / 1461) );            /* 1461 = 3 * 365 + 366 */
    days %= 1461;
    if(days >= 366)
    {
        days -= 366;
        year ++;
    }
    if(days >= 365)
    {
        days -= 365;
        year ++;
    }
    if(days >= 365)
    {
        days -= 365;
        year ++;
    }
    /* Computing year, now we are going to compute month. */
    month = 1;
    days++;
    if(days >= 32)    /* January */
    {
        days -= 31;
        month ++;
        if( (year % 4) == 0)   /* leap year */
        {
            if(days >= 30)
            {
                days -= 29;
                month ++;
                if(days >= 32)    /* March */
                {
                    days -= 31;
                    month ++;
                    if(days >= 31)    /* April */
                    {
                        days -= 30;
                        month ++;
                        if(days >= 32)    /* May */
                        {
                            days -= 31;
                            month ++;
                            if(days >= 31)    /* June */
                            {
                                days -= 30;
                                month ++;
                                if(days >= 32)    /* July */
                                {
                                    days -= 31;
                                    month ++;
                                    if(days >= 32)    /* aot */
                                    {
                                        days -= 31;
                                        month ++;
                                        if(days >= 31)    /* September */
                                        {
                                            days -= 30;
                                            month ++;
                                            if(days >= 32)    /* October */
                                            {
                                                days -= 31;
                                                month ++;
                                                if(days >= 31)    /* November */
                                                {
                                                    days -= 30;
                                                    month ++;
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        else
        {
            if(days >= 29)
            {
                days -= 28;
                month ++;
                if(days >= 32)    /* March */
                {
                    days -= 31;
                    month ++;
                    if(days >= 31)    /* April */
                    {
                        days -= 30;
                        month ++;
                        if(days >= 32)    /* May */
                        {
                            days -= 31;
                            month ++;
                            if(days >= 31)    /* June */
                            {
                                days -= 30;
                                month ++;
                                if(days >= 32)    /* July */
                                {
                                    days -= 31;
                                    month ++;
                                    if(days >= 32)    /* August */
                                    {
                                        days -= 31;
                                        month ++;
                                        if(days >= 31)    /* September */
                                        {
                                            days -= 30;
                                            month ++;
                                            if(days >= 32)    /* October */
                                            {
                                                days -= 31;
                                                month ++;
                                                if(days >= 31)    /* November */
                                                {
                                                    days -= 30;
                                                    month ++;
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    date[0] = ((year % 100) / 10) + '0';
    date[1] = (year % 10) + '0';
    date[2] = (month / 10) + '0';
    date[3] = (month % 10) + '0';
    date[4] = (days / 10) + '0';
    date[5] = (days % 10) + '0';
    date[6] = 0;
}

int is_date_valid(const char *data_string)
/* This function is used to check a date in following format YYMMDD.
Return FALSE when the date is not correct and TRUE in other case. */
{
int year, month, day;
char buffer[3];
    if( (data_string[0] < '0') || (data_string[0] > '9') )
        return false;
    if( (data_string[1] < '0') || (data_string[1] > '9') )
        return false;
    if( (data_string[2] < '0') || (data_string[2] > '1') )
        return false;
    if( (data_string[3] < '0') || (data_string[3] > '9') )
        return false;
    if( (data_string[4] < '0') || (data_string[4] > '3') )
        return false;
    if( (data_string[5] < '0') || (data_string[5] > '9') )
        return false;
    year = atoi(strncpy0(buffer, &data_string[0], 2) );
    month  = atoi(strncpy0(buffer, &data_string[2], 2) );
    day  = atoi(strncpy0(buffer, &data_string[4], 2) );
    if(day < 1)
        return false;
    if( (month < 1) || (month > 12) )
        return false;
    else
    {
        switch(month)
        {
            case  1:          /*      January         */
            case  3:          /*      March           */
            case  5:          /*      May             */
            case  7:          /*      July            */
            case  8:          /*      August          */
            case 10:          /*      October         */
            case 12:          /*      December        */
                return (day > 31) ? false : true;
            case  4:          /*      April           */
            case  6:          /*      June            */
            case  9:          /*      September       */
            case 11:          /*      November        */
                return (day > 30) ? false : true;

            default:          /*      February         */
                if(year % 4 == 0)     /* Attention : 2000 est bissextile ! */
                    return (day > 29) ? false : true;
                else
                    return (day > 28) ? false : true;
        }  /*      du switch month  */
    }     /*      du else if month < 1 ou > 12     */
}
